@@ -42,6 +42,7 @@
 ;;; Code:
 
 (require 'ebdb)
+(require 'ebdb-i18n-basic)
 
 (defgroup ebdb-i18n nil
   "Options for EBDB internationalization."
@@ -335,6 +336,10 @@ English version for display."
 English, and a three-letter symbol identifying the country, as
 per ISO 3166-1 alpha 3.")
 
+(defsubst ebdb-i18n-countries ()
+  (append ebdb-i18n-countries-pref-scripts
+	  ebdb-i18n-countries))
+
 ;; Taken from https://en.wikipedia.org/wiki/Telephone_country_codes,
 ;; on Jul 30, 2016
 (defvar ebdb-i18n-phone-codes
@@ -658,24 +663,34 @@ This method should return a new instance of CLASS.")
   address fields.
 
 Essentially this just means swapping out the string country names
-for their symbol representations.")
+for their symbol representations."
+  (let ((count 0))
+    (dolist (rec (ebdb-records))
+      (dolist (adr (ebdb-record-address rec))
+	(when (stringp (slot-value adr 'country))
+	  (ignore-errors
+	   (ebdb-record-change-field
+	    rec adr
+	    (clone adr :country
+		   (cdr (assoc-string (slot-value adr 'country)
+				      (ebdb-i18n-countries)))))
+	   (cl-incf count)))))
+    (message "Internationalized %d addresses" count)))
 
 (cl-defmethod ebdb-read :extra "i18n" ((class (subclass ebdb-field-address))
 				       &optional slots obj)
-  (let* ((countries (append ebdb-i18n-countries-pref-scripts
-			    ebdb-i18n-countries))
-	 (country
+  (let ((country
 	  (cdr (assoc (completing-read
 		       "Country: "
-		       countries
+		       (ebdb-i18n-countries)
 		       nil nil
 		       (when obj (car (rassoc (ebdb-address-country obj)
-					      countries))))
-		      ebdb-i18n-countries))))
+					      (ebdb-i18n-countries)))))
+		      (ebdb-i18n-countries)))))
     (setq slots
 	  (condition-case nil
-	      (ebdb-read-i18n class
-			      (plist-put slots :country country) obj country)
+	      (ebdb-read-i18n class country
+			      (plist-put slots :country country) obj)
 	    (cl-no-method
 	     (plist-put slots :country country))))
     (cl-call-next-method class slots obj)))
@@ -690,34 +705,30 @@ for their symbol representations.")
 	       (cl-no-method nil)))
 	(cl-call-next-method))))
 
-;; This method can be used later.  Probably there will never be a
-;; non-i18n `ebdb-parse' method for addresses, because it's just too
-;; hard to guess if you don't know the country.  At some point there
-;; should be an option called `ebdb-read-address-articulate': if that
-;; is nil, then we should require the user to have loaded `ebdb-i18n',
-;; and only international parse methods will be considered.
+;; This is not an :extra method, because there will never be a
+;; non-i18n `ebdb-parse' method for addresses.  It's just too hard to
+;; guess if you don't know the country.  This is only used in
+;; snarfing.
 
-;; (cl-defmethod ebdb-parse :extra "i18n" ((class (subclass ebdb-field-address))
-;; 					(str string)
-;; 					&optional slots)
-;;   "Internationally-aware version of `ebdb-parse' for addresses."
-;;   (let ((cc (or (plist-get slots :country)
-;; 		(when (string-match (regexp-opt
-;; 				     (mapcar
-;; 				      (lambda (elt) (car elt))
-;; 				      (append ebdb-i18n-countries-pref-scripts
-;; 					      ebdb-i18n-countries)))
-;; 				    str)
-;; 		  (cdr-safe (assoc-string
-;; 			     (match-string 0 str)
-;; 			     (append ebdb-i18n-countries-pref-scripts
-;; 				     ebdb-i18n-countries)))))))
-;;     (or (and cc
-;; 	     (symbolp cc)
-;; 	     (condition-case nil
-;; 		 (ebdb-parse-i18n class str cc slots)
-;; 	       (cl-no-method nil)))
-;; 	(cl-call-next-method))))
+(cl-defmethod ebdb-parse ((class (subclass ebdb-field-address))
+			  (str string)
+			  &optional slots)
+  "Internationally-aware version of `ebdb-parse' for addresses."
+  (let ((cc (or (plist-get slots :country)
+		(when (string-match (regexp-opt
+				     (mapcar
+				      (lambda (elt) (car elt))
+				      (ebdb-i18n-countries)))
+				    str)
+		  (cdr-safe (assoc-string
+			     (match-string 0 str)
+			     (ebdb-i18n-countries)))))))
+    (or (and cc
+	     (symbolp cc)
+	     (condition-case nil
+		 (ebdb-parse-i18n class str cc slots)
+	       (cl-no-method nil)))
+	(signal 'ebdb-unparseable (list str)))))
 
 (cl-defmethod ebdb-read :extra "i18n" ((class (subclass ebdb-field-phone))
 				       &optional slots obj)
@@ -728,7 +739,7 @@ for their symbol representations.")
 			"Country/Region: "
 			ebdb-i18n-phone-codes nil nil)
 		       ebdb-i18n-phone-codes))))
-	area-code)
+	(area-code (when obj (slot-value obj 'area-code))))
     ;; Obviously this whole structure thing is just poorly
     ;; thought-out.
     (when (consp country)
