@@ -26,8 +26,24 @@
 
 (require 'ebdb)
 (autoload 'calendar-absolute-from-gregorian "calendar")
+(autoload 'calendar-gregorian-from-absolute "calendar")
+(autoload 'org-time-string-to-absolute "org")
+(declare-function make-gnorb-ebdb-link "ext:gnorb-ebdb")
+(declare-function org-contacts-db "ext:org-contacts")
+(defvar url-handler-regexp)
 ;;; Migrating the EBDB
 
+;; For org-contacts
+
+(defvar org-contacts-email-property)
+(defvar org-contacts-alias-property)
+(defvar org-contacts-tel-property)
+(defvar org-contacts-address-property)
+(defvar org-contacts-birthday-property)
+(defvar org-contacts-note-property)
+(defvar org-contacts-icon-property)
+(defvar org-contacts-nickname-property)
+
 ;; Unused
 (defconst ebdb-migration-features
   '((3 . "* Date format for `creation-date' and `timestamp' has changed,
@@ -348,6 +364,11 @@ from the list in question."
   :group 'ebdb-mua-gnus-splitting
   :type  'symbol)
 
+(defcustom bbdb/vm-auto-folder-field 'vm-folder
+  "The name of the VM-specific xfield for mail splitting."
+  :group 'ebdb-mua-vm
+  :type 'symbol)
+
 (defcustom bbdb/gnus-score-field 'gnus-score
   "This variable contains the name of the EBDB field which should be
 checked for a score to add to the mail addresses in the same record."
@@ -360,29 +381,35 @@ checked for a score to add to the mail addresses in the same record."
   :type 'symbol)
 
 ;;;###autoload
-(defun ebdb-migrate-from-bbdb ()
+(defun ebdb-migrate-from-bbdb (&optional file)
   "Migrate from BBDB to EBDB.
 
 This upgrade is extreme enough that we can't really use the
 existing migration mechanisms.  They are still there, though, in
 case someone's going from, say, version 2 to 4 in one jump.
 
-Assume that the variable `bbdb-file' points to an existing file
-holding valid contacts in a previous BBDB format."
+Migrate from FILE, if non-nil.  Otherwise, assume that the
+variable `bbdb-file' points to an existing file holding valid
+contacts in a previous BBDB format.  If that variable isn't set
+use \(locate-user-emacs-file \"bbdb\" \".bbdb\"\), which is how
+BBDB sets the default of that option."
   (require 'url-handlers)
   (require 'ebdb-org)
   (require 'ebdb-gnus)
-  (with-current-buffer (find-file-noselect bbdb-file)
+  (with-current-buffer (find-file-noselect
+			(or file
+			    (and (bound-and-true-p bbdb-file)
+				 bbdb-file)
+			    (locate-user-emacs-file "bbdb" ".bbdb")))
     (when (and (/= (point-min) (point-max))
 	       (yes-or-no-p "Upgrade from previous version of BBDB? "))
       (let ((v-records (ebdb-migrate-parse-records))
-	    (target-db (if (= (length ebdb-db-list) 1)
-			   (car ebdb-db-list)
-			 (ebdb-prompt-for-db)))
+	    (target-db (ebdb-prompt-for-db nil t))
+	    (total 0)
 	    c-records duds)
-	(message "Migrating records...")
+	(message "Migrating records from BBDB...")
 	(dolist (r v-records)
-	  (condition-case err
+	  (condition-case-unless-debug err
 	      (let ((orgs (ebdb-vrecord-organization r))
 		    (c-rec (ebdb-migrate-vector-to-class r))
 		    org)
@@ -411,7 +438,8 @@ holding valid contacts in a previous BBDB format."
 		      (ebdb-record-add-org-role c-rec org)
 		      (unless (member org c-records)
 			(push org c-records)))))
-		(push c-rec c-records))
+		(push c-rec c-records)
+		(message "Migrating records from BBDB... %d" (cl-incf total)))
 	    (error
 	     (push (list r err) duds))))
 	(when duds
@@ -428,12 +456,12 @@ holding valid contacts in a previous BBDB format."
 	  (goto-char (point-min)))
 	(dolist (r c-records)
 	  (ebdb-init-record r))
-	(eieio-oset target-db 'dirty t)
-	(message "Migrating records... %d records migrated" (length c-records))))))
+	(setf (slot-value target-db 'dirty) t)
+	(message "Migrating records from BBDB... %d records migrated"
+		 (length c-records))))))
 
 (defun ebdb-migrate-vector-to-class (v)
-  "Migrate a single vector-style BBDB record to the EIEIO class
-  style used in EBDB."
+  "Migrate a single vector-style BBDB record V to EBDB style EIEIO object."
   ;; In the vector version, vector elements were:
 
   ;; record: firstname lastname affix aka organization phone address mail xfields
@@ -485,11 +513,6 @@ holding valid contacts in a previous BBDB format."
 	      (postcode (aref a 4))
 	      (country (aref a 5))
 	      (case-fold-search t))
-	  (when (and country (stringp country))
-	    (dolist (c ebdb-country-list)
-	      (when (and (stringp country)
-			 (string-match-p country (car c)))
-		(setq country (cdr c)))))
 	  (push (make-instance ebdb-default-address-class
 			       :object-name label
 			       :streets streets
@@ -507,7 +530,7 @@ holding valid contacts in a previous BBDB format."
 				 :mail (cadr bits))
 		  mails))))
       (when mails
-       (oset (car (last mails)) priority 'primary)))
+	(setf (slot-value (car (last mails)) 'priority) 'primary)))
     (when xfields
       (dolist (x xfields)
 	(setq lab (car x)
@@ -577,22 +600,15 @@ holding valid contacts in a previous BBDB format."
 					 val
 				       (split-string val ";" t " ")))
 		fields))
-	 ((memq lab (list bbdb/gnus-score-field
-			  bbdb/gnus-split-private-field
-			  'imap))
-	  (cond
-	   ((eq lab bbdb/gnus-score-field)
-	    (push (make-instance 'ebdb-gnus-score-field
-				 :score val)
-		  fields))
-	   ((eq lab bbdb/gnus-split-private-field)
-	    (push (make-instance 'ebdb-gnus-private-field
-				 :group val)
-		  fields))
-	   ((eq lab 'imap)
-	    (push (make-instance 'ebdb-gnus-imap-field
-				 :group val)
-		  fields))))
+	 ((eq lab bbdb/gnus-score-field)
+	  (push (make-instance 'ebdb-gnus-score-field
+			       :score val)
+		fields))
+	 ((memq lab (list 'imap bbdb/gnus-split-private-field
+			  bbdb/vm-auto-folder-field))
+	  (push (make-instance 'ebdb-field-mail-folder
+			       :folder val)
+		fields))
 	 (t
 	  (push (make-instance 'ebdb-field-user-simple
 			       :object-name (symbol-name (car x))
@@ -654,5 +670,69 @@ holding valid contacts in a previous BBDB format."
 
 	records))))
 
+(defvar ebdb-migrate-org-simple-correspondences
+  '((org-contacts-email-property . ebdb-default-mail-class)
+    (org-contacts-tel-property . ebdb-default-phone-class)
+    (org-contacts-note-property . ebdb-default-notes-class)
+    (org-contacts-nickname-property . ebdb-field-name-simple)
+    (org-contacts-alias-property . ebdb-default-name-class))
+  "The simplest property-to-field correspondences.
+This variable only holds correspondences for fields that require
+no processing beyond calling `ebdb-parse' on the string values.")
+
+;;;###autoload
+(defun ebdb-migrate-from-org-contacts ()
+  "Migrate contacts from the org-contacts format."
+  (interactive)
+  (require 'org-contacts)
+  (unless ebdb-sources
+    (error "First set `ebdb-sources' to the location of your EBDB database."))
+  (let ((db (ebdb-prompt-for-db))
+	;; Postpone evaluation of the symbols until run time.
+	(prop-fields
+	 (mapcar
+	  (pcase-lambda (`(,prop . ,class))
+	    (cons (symbol-value prop)
+		  (if (class-p class) class (symbol-value class))))
+	  ebdb-migrate-org-simple-correspondences))
+	(count 0)
+	(address-buffer (get-buffer-create "*EBDB Migration Addresses*"))
+	record records f)
+    (with-current-buffer address-buffer
+      (erase-buffer))
+    (message "Migrating records from org-contacts... %d records" count)
+    (pcase-dolist (`(,name ,_ ,fields) (org-contacts-db))
+      (setq record (make-instance ebdb-default-record-class))
+      (ebdb-record-change-name record name)
+      (dolist (field fields)
+	(setq f
+	      (if (assoc-string (car field) prop-fields)
+		  (ebdb-parse (cdr (assoc-string (car field) prop-fields))
+			      (cdr field))
+		(pcase (car field)
+		  ((pred (equal org-contacts-address-property))
+		   (with-current-buffer address-buffer
+		     (insert (format "%s: %s\n" name (cdr field)))))
+		  ((pred (equal org-contacts-birthday-property))
+		   (make-instance 'ebdb-field-anniversary
+				  :date  (calendar-gregorian-from-absolute
+					  (org-time-string-to-absolute
+					   (cdr field)))
+				  :object-name "birthday")))))
+	(when f
+	  (ebdb-record-insert-field record f)))
+      (push record records)
+      (message "Migrating records from org-contacts... %d records"
+	       (cl-incf count)))
+    (dolist (r records)
+      (ebdb-db-add-record db r))
+    (message "Migrating records from org-contacts... %d records"
+	     (length records)))
+  (ebdb-display-records records)
+  (with-current-buffer address-buffer
+    (unless (= (point-min) (point-max))
+      (pop-to-buffer address-buffer
+		     '(display-buffer-pop-up-window . ((width . 50)))))))
+
 (provide 'ebdb-migrate)
 ;;; ebdb-migrate.el ends here
