@@ -223,7 +223,7 @@ accordingly."
   (cond ((stringp ebdb-user-mail-address-re)
 	 ebdb-user-mail-address-re)
 	((eq ebdb-user-mail-address-re 'self)
-	 (let ((self-rec (ebdb-gethash ebdb-record-self 'uuid)))
+	 (let ((self-rec (ebdb-record-self)))
 	   (unless self-rec
 	     (user-error "`ebdb-user-mail-address-re' set to 'self, but `ebdb-record-self' is not set."))
 	   (setq ebdb-user-mail-address-re
@@ -483,10 +483,81 @@ Currently no other MUAs support this EBDB feature."
   :group 'ebdb-mua
   :type 'string)
 
+(defvar ebdb-mail-folder-list nil
+  "Variable holding lists of mail folder names and mail regexps.
+This is a list of lists: the car of each list element is a string
+folder name, followed by an arbitrary number of strings
+representing regular expressions matching mail addresses.
+
+The value of this variable is usually constructed from instances
+of the `ebdb-field-mail-folder' field.  It's also possible to
+manually add regexps to this list, if for instance the user
+wishes to match mail addresses more broadly.  In this case the
+variable should be set before EBDB is loaded.")
+
+(defclass ebdb-field-mail-folder (ebdb-field-user)
+  ((folder
+    :type string
+    :initarg :folder
+    :custom string
+    :documentation "The folder name to split mail to."))
+  :human-readable "mail folder"
+  :documentation "A field holding the string names of MUA
+  folders.  The MUA packages may perform automatic splitting and
+  filing of messages from records based on the value of this
+  field.")
+
+(cl-defmethod ebdb-string ((f ebdb-field-mail-folder))
+  (slot-value f 'folder))
+
+(cl-defmethod ebdb-read ((c (subclass ebdb-field-mail-folder))
+			 &optional slots obj)
+  (unless (plist-get slots :folder)
+    (setq slots (plist-put slots :folder
+			   (ebdb-read-string
+			    "Folder name: "
+			    (when obj (slot-value obj 'folder))
+			    ebdb-mail-folder-list))))
+  (cl-call-next-method c slots obj))
+
+(cl-defmethod ebdb-parse ((c (subclass ebdb-field-mail-folder))
+			  (str string)
+			  &optional slots)
+  (unless (plist-get slots :folder)
+    (setq slots (plist-put slots :folder str)))
+  (cl-call-next-method c str slots))
+
+(cl-defmethod ebdb-init-field ((f ebdb-field-mail-folder)
+			       &optional record)
+  (when record
+    (let* ((folder (slot-value f 'folder))
+	   (mails (mapcar #'regexp-quote (ebdb-record-mail-canon record)))
+	   (entry (assoc-string folder
+				ebdb-mail-folder-list)))
+      (when mails
+	(if entry
+	    (setcdr (assoc folder ebdb-mail-folder-list)
+		    (delete-dups (append (cdr entry) mails)))
+	  (push (cons folder mails)
+		ebdb-mail-folder-list)))))
+  (cl-call-next-method))
+
+(cl-defmethod ebdb-delete-field ((f ebdb-field-mail-folder)
+				 &optional record _unload)
+  (when record
+    (let* ((folder (slot-value f 'folder))
+	   (mails (mapcar #'regexp-quote (ebdb-record-mail-canon record)))
+	   (entry (assoc-string folder
+				ebdb-mail-folder-list)))
+      (when (and mails entry)
+	(setcdr (assoc folder ebdb-mail-folder-list)
+		(seq-difference (cdr entry) mails)))))
+  (cl-call-next-method))
+
 (defsubst ebdb-message-header-re (header regexp)
   "Return non-nil if REGEXP matches value of HEADER."
   (let ((val (ebdb-mua-message-header header))
-        (case-fold-search t)) ; RW: Is this what we want?
+        (case-fold-search t))		; RW: Is this what we want?
     (and val (string-match regexp val))))
 
 (defsubst ebdb-mua-check-header (header-type address-parts &optional invert)
@@ -744,9 +815,6 @@ Return the records matching ADDRESS or nil."
 	 (record-class (if (eql (nth 3 address) 'organization)
 			   'ebdb-record-organization
 			 ebdb-default-record-class))
-	 (name-class (if (eql record-class 'ebdb-record-organization)
-			 'ebdb-field-name-simple
-		       ebdb-default-name-class))
          (records (ebdb-message-search name mail))
          created-p new-records)
     (if (and (not records) (functionp update-p))
@@ -784,8 +852,7 @@ Return the records matching ADDRESS or nil."
 
         ;; Analyze the name part of the record.
         (cond (created-p		; new record
-               (ebdb-record-change-name
-		record (ebdb-parse name-class name)))
+               (ebdb-record-change-name record name))
 
               ((or (not name)
                    ;; The following tests can differ for more complicated names
@@ -816,8 +883,7 @@ Return the records matching ADDRESS or nil."
                                      (format "Keep name \"%s\" as an AKA? " old-name))
                      (ebdb-record-insert-field
                       record (slot-value record 'name) 'aka)))
-               (ebdb-record-change-name
-		record (ebdb-parse name-class name))
+               (ebdb-record-change-name record name)
                (setq change-p 'name))
 
               ;; make new name an AKA?
@@ -882,8 +948,7 @@ Return the records matching ADDRESS or nil."
                         (progn
                           (setq record (make-instance ebdb-default-record-class))
 			  (ebdb-db-add-record (car ebdb-db-list) record)
-                          (ebdb-record-change-name
-			   record (ebdb-parse name-class name))
+                          (ebdb-record-change-name record name)
                           (setq created-p t))))
 
                (let ((mails (ebdb-record-mail record)))
@@ -1067,6 +1132,15 @@ where it was in the MUA, rather than quitting the EBDB buffer."
 	(equal (this-command-keys-vector)
 	       [?q]))))))
 
+;;;###autoload
+(defun ebdb-mua-toggle-records-format ()
+  "Toggle format of all records without leaving MUA."
+  (interactive)
+  (let ((buf (get-buffer (ebdb-make-buffer-name))))
+    (when buf
+     (with-current-buffer buf
+       (ebdb-toggle-records-format ebdb-records)))))
+
 ;;;###autoload
 (defun ebdb-mua-edit-sender-notes ()
   "Edit the notes field of the EBDB record of the message sender."
@@ -1144,12 +1218,13 @@ buffer."
 				  (with-current-buffer b
 				    (derived-mode-p 'ebdb-mode)))
 				(buffer-list))))))
+	 mail
 	 (addresses
 	  (with-current-buffer buffer
             (delq nil
                   (mapcar (lambda (x)
-                            (when-let ((mail (car (ebdb-record-mail (car x) t))))
-                              (ebdb-dwim-mail (car x) mail)))
+			    (when (setq mail (car (ebdb-record-mail (car x) t)))
+			      (ebdb-dwim-mail (car x) mail)))
                           ebdb-records)))))
     (if (derived-mode-p 'message-mode 'mail-mode)
 	(when addresses
@@ -1181,6 +1256,8 @@ If `ebdb-mua-pop-up' is non-nil, EBDB pops up the *EBDB* buffer
 along with the MUA window(s), displaying the matching records."
   (let* ((ebdb-silent-internal t)
 	 records)
+    (when (null ebdb-record-tracker)
+      (ebdb-load))
     (setq records (ebdb-update-records
 		   (ebdb-get-address-components header-class)
 		   (or update-p
@@ -1204,6 +1281,7 @@ along with the MUA window(s), displaying the matching records."
     (define-key km (kbd "'") #'ebdb-mua-edit-sender-notes)
     (define-key km (kbd "\"") #'ebdb-mua-in-ebdb-buffer)
     (define-key km (kbd "s") #'ebdb-mua-snarf-article)
+    (define-key km (kbd "t") #'ebdb-mua-toggle-records-format)
     km)
   "Common keymap for calling EBDB commands in an MUA.
 
